<?php

namespace LakeDrops\DockerTraefik;

use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\Yaml\Yaml;

/**
 * Class Traefik.
 *
 * @package LakeDrops\DockerTraefik
 */
class Traefik {

  /**
   * The name of the project using Traefik.
   *
   * @var string
   */
  protected $name;

  /**
   * @var string
   */
  protected $domain;

  /**
   * @var int
   */
  protected $http_port;

  /**
   * @var int
   */
  protected $https_port;

  /**
   * @var string
   */
  protected $cert_filename;

  /**
   * @var string
   */
  protected $key_filename;

  /**
   * Traefik constructor.
   *
   * @param string $name
   *   Name of the network to be created which needs to match the container
   *   prefix of your project you would like to handle with Traefik.
   * @param string $domain
   *   The domain name used for all local projects.
   * @param int $http_port
   *   Port for non secure requests.
   * @param int $https_port
   *   Port for secure requests.
   * @param string $cert_filename
   *   Filename of the SSL certificate.
   * @param string $key_filename
   *   Filename of the private key for the SSL certificate.
   */
  public function __construct(string $name, $domain = 'docker.localhost', $http_port = 8000, $https_port = 8443, $cert_filename = 'fullchain.pem', $key_filename = 'privkey.pem') {
    $this->name = $name;
    $this->domain = $domain;
    $this->http_port = $http_port;
    $this->https_port = $https_port;
    $this->cert_filename = $cert_filename;
    $this->key_filename = $key_filename;
  }

  /**
   * Update the Traefik container.
   *
   * @param bool $rewrite
   */
  public function update($rewrite = FALSE): void {
    // Update host wider traefik container.
    $traefikPath = $_SERVER['HOME'] . '/.traefik';
    $traefikCertPath = $_SERVER['HOME'] . '/.traefik/certs';
    $traefikConfigPath = $_SERVER['HOME'] . '/.traefik/configuration';
    $traefikFile = $traefikPath . '/docker-compose.yml';

    $fs = new Filesystem();
    if (!$rewrite && $fs->exists($traefikFile)) {
      $traefik = Yaml::parse(file_get_contents($traefikFile));
    }
    else {
      $fs->mkdir($traefikPath);
      $traefik = $this->defaultDockerCompose();
    }
    if (!$fs->exists($traefikCertPath)) {
      $fs->mkdir($traefikCertPath);
    }
    if (!$fs->exists($traefikConfigPath)) {
      $fs->mkdir($traefikConfigPath);
    }
    file_put_contents($traefikConfigPath . '/certificates.toml', $this->defaultCertificatesConfig());

    if (!in_array($this->name, $traefik['services']['traefik']['networks'], TRUE)) {
      $traefik['services']['traefik']['networks'][] = $this->name;
      $traefik['networks'][$this->name] = [
        'driver' => 'bridge',
      ];
      file_put_contents($traefikFile, Yaml::dump($traefik, 9, 2));
      $cwd = getcwd();
      chdir($traefikPath);
      exec('docker-compose --project-name traefik up -d');
      chdir($cwd);
    }
  }

  /**
   * @return array
   */
  private function defaultDockerCompose(): array {
    return [
      'version' => '3',
      'services' => [
        'traefik' => [
          'image' => 'traefik:v2.3',
          'command' => [
            '--api=true',
            '--api.dashboard=true',
            '--api.insecure=true',
            '--entrypoints.web.address=:80',
            '--entrypoints.websecure.address=:443',
            '--entrypoints.websecure.http.tls.domains[0].main=' . $this->domain,
            '--entrypoints.websecure.http.tls.domains[0].sans=.' . $this->domain,
            '--providers.file.directory=/configuration/',
            '--providers.file.watch=true',
            '--providers.docker=true',
            '--providers.docker.exposedbydefault=false',
          ],
          'restart' => 'unless-stopped',
          'networks' => [
            'internal',
          ],
          'ports' => [
            $this->http_port . ':80',
            $this->https_port . ':443',
          ],
          'labels' => [
            'traefik.enable=true',
            'traefik.network=internal',
            'traefik.http.routers.traefik.service=api@internal',
            'traefik.http.routers.traefik.rule=Host(`traefik.' . $this->domain . '`)',
          ],
          'volumes' => [
            './certs:/certs/:ro',
            './configuration:/configuration/:ro',
            '/var/run/docker.sock:/var/run/docker.sock:ro',
          ],
        ],
      ],
      'networks' => [
        'internal' => [
          'internal' => true,
        ],
      ],
    ];
  }

  /**
   * @return string
   */
  private function defaultCertificatesConfig(): string {
    return <<<EOF
[[tls.certificates]]
   certFile = "/certs/$this->cert_filename"
   keyFile = "/certs/$this->key_filename"
EOF;

  }

}
